<?php

namespace Modules\DF\Controllers;

use Modules\DF\Services\Lancamento as Service;
use Controllers\LogController;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request\DataSet;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\Remote\cURLRequest;
use Zeedhi\Framework\Remote\Exception;
use Zeedhi\Framework\Remote\HttpInterface;
use Zeedhi\Framework\Remote\FileUploader\Server as Server;
use Zeedhi\Framework\Remote\FileUploader\Listener as Listener;
use Teknisa\ZeedhiBundle\Remote\FileUploader\StaticApiKeyProvider as StaticApiKeyProvider;
use Teknisa\ZeedhiBundle\Remote\FileUploader\Factory as FileUploaderFactory;
use Zeedhi\Framework\DataSource\FilterCriteria as FilterCriteria;
use Teknisa\Libs\Util\Utilities;

class Lancamento{
    
    const REMOTE_FILE_FIELD_NAME = 'FILE';
    const REMOTE_FOLDER_FIELD_NAME = 'FOLDER';
    const REQUEST_DATA_SET = 'upload';
    const REQUEST_TRANSMISSAO_DATA_SET = 'config_transmissao';
    const RESPONSE_DATA_SET = 'new_files';
    const UPLOAD_ROUTE = '/upload';
    const DELETE_ROUTE = '/delete';
    const DOWNLOAD_ROUTE = '/files';
    
    protected $service;
    protected $devolucao = false;
    protected $nflancada = false;
    protected $devolucaoLancada = false;
    protected $token = null;
    protected $LogController;
    
    public function __construct(Service $service, $LogController, $session, $environment, $upload_routes, $linkTransmissaoDf) {
        $this->service = $service;
        $this->logController = $LogController;
        $this->environment = $environment;
        $this->session = $session;
        $this->routesToListen = $upload_routes;
        $this->linkTransmissaoFebos = $linkTransmissaoDf;
        $integration = json_decode(file_get_contents(__DIR__ . '/../../../../../backend/vendor/teknisa/libraries-next/config/integration.json'), true);
        $this->clientSecret = $integration[0]['clientSecret'];
        $this->clientId = $integration[0]['clientId'];
        $this->PRODUCTION_SERVER_URL = 'https://midia.teknisa.com/index.php';
        $this->DEV_SERVER_URL = 'http://midiatest.teknisa.com/index.php';
        $this->factoryMethod = "factoryProductionServer";
    }
    
    public function entry(Request $request, Response $response){
        $start = date("H:i:s");
        if(empty($_POST)){
			$input = file_get_contents('php://input');
			$requestHTTP = json_decode($input, true); 
		}else{
			$requestHTTP = $_POST;
		}
		
        if(is_null($requestHTTP)){
            return $this->getResponse("E", ["Nenhuma nota enviada."], [], $response);
        }
        if(is_null($requestHTTP["Produtos"]) || empty($requestHTTP["Produtos"])){
                $returnResponse = $this->getResponse("E", ["Nenhum produto enviado."], [], $response);
        }
        
        $tax = $requestHTTP;
        $products = $tax["Produtos"];
        
        if(isset($tax["Encargos"])){
           $charges = $tax["Encargos"]; 
        }else{
           $charges = array(); 
        }
        if(isset($tax["DadosAdicionais"])){
            $dacs = $tax["DadosAdicionais"];
        }else{
            $dacs = array();
        }
        $primaryKeys = $this->service->getPrimaryKeys($tax);
        
            
        if(sizeof($primaryKeys["errors"]) > 0){
            return $returnResponse = $this->getResponse("E", $primaryKeys["errors"], [], $response);
        }else{
            $tax = $primaryKeys["taxUpdated"];
        }
        $this->logController->saveMasterLog();
        try{
            $this->logController->saveMasterLog();
        }catch(\Exception $e){
            return $this->getResponse("E", [$e->getMessage()], [], $response);
        }
        
        try{
            $validationEntry = $this->validateKeys($tax, $products, $charges, $dacs);
        }catch(\Exception $e){
            return $this->getResponse("E", [$e->getMessage()], [], $response);
        }
        
        if($validationEntry["SuccessValidation"] == true){
            $tax = $validationEntry["newTax"];
            $returnEntry = $this->entryTax($tax);
            if($returnEntry["SuccessEntry"] == true){
                $tax = $returnEntry["taxUpdated"];
                $returnCharges = $this->service->updateCharges($tax["Encargos"], $tax["CDFILIAL"], $tax["NRLANCTONF"]);
                if($returnCharges["SuccessCharge"] == true){
                    $productsSave = $this->service->saveProducts(
                                                                    $tax["NRAUTOFORN"],
                                                                    $tax["Produtos"], 
                                                                    $tax["CDEMPRESA"], 
                                                                    $tax["CDINSCESTA"], 
                                                                    $tax["CDTIPOOPER"], 
                                                                    $tax["CDFORNECED"], 
                                                                    $tax["CDFILIAL"], 
                                                                    $tax["NRLANCTONF"], 
                                                                    $tax["DTENTRSAID"], 
                                                                    $tax["CDOPERADOR"], 
                                                                    $tax["IDORIGDESTNF"], 
                                                                    $tax["IDENTRSAIDOP"],
                                                                    $tax["CDGRUPOFISCAL"], 
                                                                    $tax["IDINTEREST"], 
                                                                    $tax["CDFILIRECE"], 
                                                                    $tax["CDFILIEMIT"]);
                    $tax["Produtos"] = $productsSave["Updated"];
                    if($productsSave["SuccessProducts"] == true){
                        $anexos = [$tax["ANEXO"], $tax["CUSTOMCLOB1"]];
                        $returnAnexos = $this->saveAnexos($request, $anexos, $tax);
                        if($returnAnexos["SuccessAnexos"] == true){
                            $returnDacs = $this->service->saveDacs($tax["DadosAdicionais"], $tax["CDFILIAL"], $tax["NRLANCTONF"]);
                            if($returnDacs["SuccessDacs"] == true){
                                $returnTot = $this->service->insertNfTot($tax["CDFILIAL"], $tax["NRLANCTONF"]);
                                if($returnTot["SuccessTot"] == true){
                                    $errors = $this->closeTax($tax);  
                                    if($errors["SuccessClose"] == true){
                                        $this->nflancada = true;
                                    }else{
                                        $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                                        return $returnResponse = $this->getResponse("E", $errors["MessagesClose"], [], $response);
                                    }
                                }else{
                                    $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                                    return $returnResponse = $this->getResponse("E", $errors["MessagesTot"], [], $response);
                                }
                            }else{
                                $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                                return $returnResponse = $this->getResponse("E", $returnDacs["MessagesDacs"], [],  $response);
                            }
                        }else{
                            $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                            return $returnResponse = $this->getResponse("E", $returnAnexos["MessagesAnexos"], [],  $response);
                        }
                    }else{
                        if($productsSave["afBaixada"]){
                            $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $productsSave["produtosBaixados"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                        }else{
                            $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"]);
                        }
                        return $returnResponse = $this->getResponse("E", $productsSave["MessagesProducts"], [],  $response);
                    }
                }else{
                    if(isset($tax["CDFILIAL"], $tax["NRLANCTONF"]))
                        $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"]);
                    return $returnResponse = $this->getResponse("E", $returnCharges["MessagesCharge"], [],  $response);
                }
            }else{
                $tax = $returnEntry["taxUpdated"];
                if(isset($tax["CDFILIAL"], $tax["NRLANCTONF"]))
                    $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"]);
                return $returnResponse = $this->getResponse("E", $returnEntry["MessagesEntry"], [],  $response);
            }
        }else{
            if(isset($tax["CDFILIAL"], $tax["NRLANCTONF"]))
                $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
            return $this->getResponse("E", $validationEntry["messagesValidation"], [], $response);
        }
        $this->devolucao = $this->service->verificaDevolucao();
        if($this->devolucao){
            try{
                $messages = $this->service->insertDevolucao($tax["CDFILIAL"], $tax["NRLANCTONF"]);
                if(sizeof($messages) > 0){
                    $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                    return $this->getResponse("E", $messages, [], $response);
                }
            }catch(\Exception $e){
                $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                return $this->getResponse("E", [$e->getMessage()], [], $response);
            }
        }
        if($this->nflancada && $tax["CDTIPOOPER"] == '01'){
            try{
                $this->service->taxApproved($tax["CDFILIAL"], $tax["NRLANCTONF"]);  
            }catch(\Exception $e){
                $this->service->deletaNf($tax["CDFILIAL"], $tax["NRLANCTONF"], $tax["Produtos"], $tax["CDTIPOOPER"], $tax["IDENTRSAIDOP"]);
                return $this->getResponse("E", [$e->getMessage()], [], $response);
            }
        }
        try{
            //cdpaisbacen 1058 = brasil // 1589 = chile
            $country = $this->service->getInfoCountry($tax["CDEMPRESA"]);
            //$country["CDPAISBACEN"] = '1589';
            if($country["CDPAISBACEN"] == '1589'){
                if($tax["CDTIPOOPER"] == '02' && $tax["IDENTRSAIDOP"] == 'S'){ 
                    $token = null; 
                    $requestType = '/lib_requestToken'; 
                    $data = array(
                        'CLIENT_ID' => $this->clientId,
                        'CLIENT_SECRET' => $this->clientSecret
                    ); 
                    $dataSetResultPos = 'token'; 
                    $privateKey = $this->clientSecret; 
                    //get Token
                    $token = $this->callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $this->linkTransmissaoFebos, false);
                    if(isset($token)){
                        if(isset($token[0])){
                            $tokenConfig = $token[0];
                            $requestTypeConfig = '/config_transmissao';
                            $dataConfig = [
                                'requestType' => 'FilterData',
                                'filter' => [
                                                [
                                                    "name" => "ROW",
                                                    "operator"=> "=",
                                                    "value"=> [
                                                            'CDEMPRESA' => $tax["CDEMPRESA"],
                                                            'CDINSCESTA' =>  $tax["CDINSCESTA"],
                                                            'CDFILIAL' => $tax["CDFILIAL"],
                                                            'IDENDEMISFIL' => $tax["IDENDEMISFIL"],
                                                            'NRORG' => $tax["NRORG"]
                                                        ],
                                                ]
                                            ]
                            ];
                            
                            $dataSetResultPosConfig = 'config_transmissao'; 
                            //get Config
                            $result = $this->callWebService($tokenConfig, $requestTypeConfig, $dataConfig, $dataSetResultPosConfig, $privateKey, $this->linkTransmissaoFebos);
                            if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('config', $result)){
                                $dataTransmissao = [
                                    'requestType' => 'FilterData',
                                    'filter' => [
                                                    [
                                                        "name" => "notasfiscais",
                                                        "operator"=> "=",
                                                        "value"=> [
                                                                    [
                                                                        "CDFILIAL" => $tax["CDFILIAL"],
                                                                        "NRLANCTONF" => $tax["NRLANCTONF"],
                                                                        "NRORG" => $tax["NRORG"]
                                                                    ]
                                                                ],
                                                    ],
                                                    [
                                                        "name" => "config",
                                                        "operator"=> "=",
                                                        "value"=> $result['config'],
                                                    ],
                                                    [
                                                        "name" => "tipo",
                                                        "operator"=> "=",
                                                        "value"=> '',
                                                    ]
                                                ],
                                ];
                                $tokenTransmissao = $tokenConfig;
                                $requestTypeTransmissao = '/transmiteNotasFebos';
                                $dataSetResultPosTransmissao = 'config_transmissao';
                                $resultTransmissao = $this->callWebService($tokenTransmissao, $requestTypeTransmissao, $dataTransmissao, $dataSetResultPosTransmissao, $privateKey, $this->linkTransmissaoFebos);
                            }
                        }
                    }
                }
            }
        }catch(\Exception $e){
        }
        $final = date("H:i:s");
        return $returnResponse = $this->getResponse("S", [], $returnEntry["KeysEntry"], $response);
    }
    
    public function validateKeys($tax, $products, $charges, $dacs){
        $returnService = $this->service->validateTax($tax, $products, $charges, $dacs);
        $messagesTax = $returnService["messages"];
        $tax = $returnService["taxAjusted"];
        
        $messagesTax = array_values($messagesTax);
        $messagesTax = !empty($messagesTax) ? $messagesTax : [];
        $messagesValidation = array_merge($messagesTax);
        
        $return = array(
            "SuccessValidation" => !(sizeof($messagesValidation) > 0),
            "messagesValidation"=> $messagesValidation,
            "newTax" => $tax
        );
        
        return $return;
    }
    
    public function entryTax($tax){
        $response = $this->service->saveTax($tax);
        $return = array(
            "SuccessEntry" => !(sizeof($response["messages"]) > 0),
            "MessagesEntry"=> $response["messages"],
            "KeysEntry" => $response["keys"],
            "taxUpdated" => $response["taxUpdated"]
        );
        
        return $return;
    }
    
    public function closeTax($tax){
        $response = $this->service->closeTax($tax);
        $return = array(
            "SuccessClose" => !(sizeof($response["messages"]) > 0),
            "MessagesClose"=> $response["messages"]
        );
        
        return $return;
    }
    
    public function getResponse($status, $messages, $keys, &$response){
        $retorno = array();
        $retorno["STATUS"] = $status;
        $retorno["MENSAGEM"] = $messages;
        $retorno["KEYS"] = $keys;
        header('Content-Type: application/json; charset=utf-8');
        $response->setMetadata($retorno);
    }
    
    //remover
    public function saveAnexos($request, $anexos, $tax){
        try{
    	    $msgsAnexos = array();
    	    $tax = $this->saveFileServer($request, $tax);
            $return = $this->service->saveAnexo($tax);
            return $return;
        }catch(\Exception $e){
            return array("SuccessAnexos" => false, "MessagesAnexos" => [$e->getMessage()], "afBaixada" => true);
        }
    }
    
    public function getToken($nrorg){
        $filter = array(
            "requestType" => "FilterData",
            "filter" => array( array(
            'name' => 'NRORG', 'operator' => '=', 'value' => $nrorg
        )));
        
        $config = Utilities::getLoginConfig();
        $apiKey = $this->callWebService(null, '/getOrgFileServerKey', $filter, 'fileServerKey', $config['loginClientSecret'], 'https://license.teknisa.com/backend/index.php');
        if($apiKey){
        return $apiKey[0];
        }else{
        return null;
        }
    }
    
    public function saveFileServer($request, $requestHTTP){
        $requestHTTP["DSLINKANEXO"] = [];
        $existeAmbos = false;
        $pdfExiste = false;
        $anexoExiste = false;
        $nameIdentificador = $requestHTTP["CDFILIAL"] . "_" . $requestHTTP["NRLANCTONF"] . "_";
        if(isset($requestHTTP["ANEXO"]) && $requestHTTP["ANEXO"] != ''){
            $pdfExiste = true;
            $dslinkAnexo = [
                "b64File" =>  "data:application/pdf;base64," . $requestHTTP["ANEXO"],
                "name"  => 'Anexo_' . $nameIdentificador . $requestHTTP["NRNOTAFISC"] . '_' . $requestHTTP["NRINSJURFORN"] . '.pdf',
                "type"  => "application/pdf",
                "size" => ''
            ];
            array_push($requestHTTP["DSLINKANEXO"], $dslinkAnexo);
        }
        if(isset($requestHTTP["CUSTOMCLOB1"]) && $requestHTTP["CUSTOMCLOB1"] != ''){
            $anexoExiste = true;
            $dslinkClob = [
                "b64File" => "data:application/pdf;base64," . $requestHTTP["CUSTOMCLOB1"],
                "name"  =>'Clob_'. $nameIdentificador . $requestHTTP["NRNOTAFISC"] . '_' . $requestHTTP["NRINSJURFORN"] . '.pdf',
                "type"  =>"application/pdf",
                "size" => ''
            ];
            array_push($requestHTTP["DSLINKANEXO"], $dslinkClob);
        }
        
        if($pdfExiste && $anexoExiste){
            $existeAmbos = true;
        }else{
            if(!$pdfExiste && !$anexoExiste){
                $requestHTTP["DSLINKANEXO"] = [];
                return $requestHTTP;
            }
        }
        
        //fileserver - token
        if($requestHTTP["NRORG"] == '1657'){ // compass chile
            $this->PRODUCTION_SERVER_URL = 'https://fileserver-qa.compass-group.cl/index.php';
            $requestHTTP["TOKEN"] = '48e3dfc768a1e5060ce17480e68963cd'; //pode ser alterado
        }else{
            $requestHTTP["TOKEN"] = $this->getToken($requestHTTP["NRORG"]);
        }
        $requestRow = null;
        $requestRow = new Row($requestHTTP, "POST", $request->getRoutePath(), $request->getUserId());
        $requestHTTP["DSLINKANEXO"] = $this->preDispatch($requestRow)[0][0];
        if($existeAmbos){
            $requestHTTP["DSLINKANEXO"][0]["b64File"] = "data:application/pdf;base64," . $requestHTTP["ANEXO"];
            $requestHTTP["DSLINKANEXO"][1]["b64File"] = "data:application/pdf;base64," . $requestHTTP["CUSTOMCLOB1"];
        }else if($pdfExiste || $anexoExiste){
            if($pdfExiste){
                $requestHTTP["DSLINKANEXO"][0]["b64File"] = "data:application/pdf;base64," . $requestHTTP["ANEXO"];
            }
            if($anexoExiste){
                $requestHTTP["DSLINKANEXO"][0]["b64File"] = "data:application/pdf;base64," . $requestHTTP["CUSTOMCLOB1"];
            }
        }
        
        return $requestHTTP;
    }
    
    protected function isUploadRoute($routePath) {
        return isset($this->routesToListen[$routePath]);
    }
    
    protected function getRequestRows(Request $request) {
        $rows = array();
        if ($request instanceof Request\Row) {
            $rows = array($request->getRow());
        }

        if ($request instanceof Request\DataSet) {
            $rows = $request->getDataSet()->getRows();
        }

        return $rows;
    }
    
    /**
     * @param $row
     * @param $fileField
     * @return bool
     */
    protected function shouldUpload($row, $fileField) {
        return isset($row[$fileField]) && is_array($row[$fileField]) && isset(current($row[$fileField])['b64File']);
    }
    
    protected function createFileServerRow($row, $fileFieldName, $customFolder) {
        return array(
            self::REMOTE_FILE_FIELD_NAME => $row[$fileFieldName],
            self::REMOTE_FOLDER_FIELD_NAME => $customFolder
        );
    }
    
    public function preDispatch(Request $request) {
        if ($this->isUploadRoute($request->getRoutePath())) {
            $route = $this->routesToListen[$request->getRoutePath()];
            $fileField = $route['fileField'];
            $customFolder = $route['customFolder'];
            $rowUpdated = array();
            foreach($this->getRequestRows($request) as $row) {
                if ($this->shouldUpload($row, $fileField)) {
                    $row = $this->uploadFilesInRow($row, $fileField, $customFolder,  $request->getUserId());
                    array_push($rowUpdated, $row);
                }
            }
            return $rowUpdated;
        }
    }
    
    protected function createResponse($responseString) {
        $requestArray = json_decode($responseString, true);
        if ($requestArray === null || !is_array($requestArray)) {
            throw ServerException::badFormattedResponse($responseString);
        }

        $dataSets =      isset($requestArray['dataset']) ? $requestArray['dataset'] : array();
        $error =         isset($requestArray['error'])  ? $requestArray['error'] : null;
        $messages =      isset($requestArray['messages']) ? $requestArray['messages'] : array();
        $notifications = isset($requestArray['notifications']) ? $requestArray['notifications'] : array();
        $methods =       isset($requestArray['method']) ? $requestArray['method'] : array();

        $responseDTO = new DTO\Response();

        $this->setDataSets($responseDTO, $dataSets);
        $this->setNotifications($responseDTO, $notifications);
        $this->setMessages($responseDTO, $messages);
        $this->setMethods($responseDTO, $methods);
        $this->setError($responseDTO, $error);

        return $responseDTO;
    }
    
    protected function normalizeFieldValue($fieldValue) {
        $newFieldValue = array();
        foreach($fieldValue as $fileData) {
            $newFieldValue[] = array(
                "name" => $fileData['NAME'],
                "size" => $fileData['SIZE'],
                "type" => $fileData['TYPE'],
                "path" => $fileData['PATH'],
            );
        }

        return $newFieldValue;
    }

    protected function checkResponseError(Response $response) {
        if ($response->getError()) {
            throw Exception::remoteServerError($response->getError());
        }
    }
    
    /**
     * Upload files in rows and replace the property to the file data with server path.
     *
     * @param array|\ArrayAccess $row           The row where files are stored.
     * @param string             $fileFieldName The name of column row to be uploaded.
     * @param string             $customFolder  The folder where files should be stored.
     *
     * @throws Exception
     */
    public function uploadFilesInRow(&$row, $fileFieldName, $customFolder = "", $userId = "") {
        $serverRow = $this->createFileServerRow($row, $fileFieldName, $customFolder);
        $dataSet = new \Zeedhi\Framework\DataSource\DataSet(self::REQUEST_DATA_SET, array($serverRow));
        $server = $this->factoryMethod == "factoryProductionServer" ? $this->factoryProductionServer($row["TOKEN"]) : $this->factoryDevServer($row["TOKEN"]);
        $response = $server->request(new DataSet($dataSet, HttpInterface::METHOD_POST, self::UPLOAD_ROUTE, $userId));
        $this->checkResponseError($response);
        $responseDataSet = $response->getDataSets()[0];
        $subRow = array();
        foreach ($responseDataSet->getRows() as $key => $responseRow) {
            $row[$fileFieldName] = $this->normalizeFieldValue($responseRow[self::REMOTE_FILE_FIELD_NAME]);
            array_push($subRow, $row[$fileFieldName]);
        }
        return $subRow;
    }
    
    protected function factoryServer($token, $url) {
        return new Server(self::getHttpStrategy($url), $token);
    }
    
    public function factoryDevServer($token) {
        return self::factoryServer($token, $this->DEV_SERVER_URL);
    }

    public function factoryProductionServer($token) {
        return self::factoryServer($token, $this->PRODUCTION_SERVER_URL);
    }
    
    protected static function getHttpStrategy($url) {
        return new cURLRequest($url);
    }
    
    public static function callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url, $keepConnected = false){
        set_time_limit(180);
        $header = array();
        $header[] = 'OAuth-Token: ' . $token;
        $header[] = 'OAuth-SecretId: ' . $privateKey;
        if($keepConnected) {
            $header[] = 'OAuth-KeepConnected: ' . $keepConnected;
        }
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen(json_encode($data));
        
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_URL, $url . $requestType);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);

        $result = curl_exec($ch);
        //var_dump($result);
        $result = json_decode($result, true);
        $error = curl_error($ch);
        curl_close($ch);

        if (!empty($result['error'])) {
            throw new \Exception($result['error']);
        } else if (!empty($error)) {
            throw new \Exception($error);
        } else {
            return isset($result['dataset'][$dataSetResultPos]) ? $result['dataset'][$dataSetResultPos] : null;
        }
    }
}